@@ -41,6 +41,7 @@
 #include <eel/eel-canvas.h>
 #include <eel/eel-canvas-util.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <math.h>
 #include <stdio.h>
 
@@ -125,6 +126,7 @@ struct EelBackgroundDetails {
 	gboolean constant_size;
 	
 	char *image_uri;
+	time_t image_mtime;
 	GdkPixbuf *image;
 	int image_width_unscaled;
 	int image_height_unscaled;
@@ -551,6 +553,7 @@ eel_background_image_totally_obscures (EelBackground *background)
 	switch (background->details->image_placement) {
 	case EEL_BACKGROUND_TILED:
 	case EEL_BACKGROUND_SCALED:
+	case EEL_BACKGROUND_ZOOM:
 		return TRUE;
 	default:
 		g_assert_not_reached ();
@@ -676,6 +679,24 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 				}
 			}
 			break;
+		case EEL_BACKGROUND_ZOOM:
+			eel_gdk_scale_to_min_factor (background->details->image_width_unscaled,
+						     background->details->image_height_unscaled,
+						     dest_width, dest_height,
+						     &fit_width, &fit_height);
+
+			if (image_width != fit_width || image_height != fit_height) {
+				if (cur_scaled) {
+					reload_image = TRUE;
+				} else {
+					scaled_pixbuf = eel_gdk_pixbuf_scale_to_min (background->details->image, dest_width, dest_height);
+					g_object_unref (background->details->image);
+					background->details->image = scaled_pixbuf;
+					image_width = gdk_pixbuf_get_width (scaled_pixbuf);
+					image_height = gdk_pixbuf_get_height (scaled_pixbuf);
+				}
+			}
+			break;
 		}
 
 		if (reload_image) {
@@ -757,6 +778,7 @@ get_pixmap_size (EelBackground    *background,
 		case EEL_BACKGROUND_CENTERED:
 		case EEL_BACKGROUND_SCALED:
 		case EEL_BACKGROUND_SCALED_ASPECT:
+		case EEL_BACKGROUND_ZOOM:
 			*pixmap_width = entire_width;
 			*pixmap_height = entire_height;
 		}
@@ -1354,6 +1376,7 @@ eel_background_draw_aa (EelBackground *background, EelCanvasBuf *buffer)
 		case EEL_BACKGROUND_CENTERED:
 		case EEL_BACKGROUND_SCALED:
 		case EEL_BACKGROUND_SCALED_ASPECT:
+		case EEL_BACKGROUND_ZOOM:
 			/* Since the image has already been scaled, all these cases
 			 * can be treated identically.
 			 */
@@ -1524,9 +1547,22 @@ eel_background_set_image_uri_helper (EelBackground *background,
 				     gboolean emit_appearance_change,
 				     gboolean load_async)
 {
+        time_t mtime;
+        GnomeVFSFileInfo *info;
+
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
 
-	if (eel_strcmp (background->details->image_uri, image_uri) == 0) {
+	mtime = background->details->image_mtime;
+	if (image_uri != NULL) {
+		info = gnome_vfs_file_info_new ();
+		if (gnome_vfs_get_file_info (image_uri, info,
+					     GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == GNOME_VFS_OK) {
+			mtime = info->mtime;
+		}
+		gnome_vfs_file_info_unref (info);
+	}
+	if (eel_strcmp (background->details->image_uri, image_uri) == 0 &&
+	    mtime == background->details->image_mtime) {
 		background->details->emit_after_load = emit_appearance_change;
 		return FALSE;
 	}
@@ -1535,6 +1571,7 @@ eel_background_set_image_uri_helper (EelBackground *background,
 	
 	g_free (background->details->image_uri);
 	background->details->image_uri = g_strdup (image_uri);
+        background->details->image_mtime = mtime;
 
 	/* We do not get rid of the current image here. This gets done after the new
 	 * image loads - in eel_background_load_image_callback. This way the
